@@ -15,7 +15,13 @@ import numpy as np
 import sys
 from scipy.spatial import Delaunay
 
-from PyQt4.QtCore import Qt, QSize, QString, SIGNAL
+import PyQt4
+from PyQt4.QtCore import Qt, QSize, SIGNAL
+try:
+    from PyQt4.QtCore import QString
+except ImportError:
+    # we are using Python3 so QString is not defined
+    QString = type("")
 from PyQt4.QtGui import QImage, QDialog,\
     QApplication, QSlider, QPushButton,\
     QLabel, QPixmap, QPainter, qRgba,\
@@ -27,12 +33,20 @@ GRAY_COLORTABLE = np.array([[ii, ii, ii, 255] for ii in range(256)],
                            dtype=np.uint8)
 
 SEEDS_COLORTABLE = np.array([[0, 255, 0, 255],
-                             [0, 0, 255, 255]], dtype=np.uint8)
+                             [64, 0, 255, 255],
+                             [0, 200, 128, 255],
+                             [64, 128, 200, 255]], dtype=np.uint8)
 
 CONTOURS_COLORS = {
     1: [255, 0, 0],
     2: [0, 0, 255],
     3: [0, 255, 255],
+    4: [255, 128, 0],
+    5: [255, 128, 0],
+    6: [255, 0, 128],
+    7: [128, 128, 255],
+    8: [0, 128, 255],
+    9: [128, 0, 255],
     10: [255, 255, 0],
     11: [0, 255, 0],
     12: [0, 255, 255],
@@ -44,7 +58,7 @@ CONTOURS_COLORTABLE[:,:3] = 255
 CONTOURLINES_COLORTABLE = np.zeros((256,2,4), dtype=np.uint8)
 CONTOURLINES_COLORTABLE[:,:,:3] = 255
 
-for ii, jj in CONTOURS_COLORS.iteritems():
+for ii, jj in CONTOURS_COLORS.items():
     key = ii - 1
     CONTOURS_COLORTABLE[key,:3] = jj
     CONTOURS_COLORTABLE[key,3] = 64
@@ -81,6 +95,8 @@ BOX_BUTTONS_SEED = {
     Qt.RightButton: 2,
     }
 
+BOX_BUTTONS_SEED_SHIFT_OFFSET = 2
+
 BOX_BUTTONS_DRAW = {
     Qt.LeftButton: 1,
     Qt.RightButton: 0,
@@ -174,6 +190,7 @@ class SliceBox(QLabel):
         xx = xx[ii]
         yy = yy[ii]
 
+        
         self.seeds[yy * self.slice_size[0] + xx] = self.seed_mark
 
     def drawLine(self, p0, p1):
@@ -385,7 +402,19 @@ class SliceBox(QLabel):
     def mousePressEvent(self, event):
         if event.button() in self.box_buttons:
             self.drawing = True
-            self.seed_mark = self.box_buttons[event.button()]
+            modifiers = PyQt4.QtGui.QApplication.keyboardModifiers()
+            shift_offset = 0
+            if modifiers == PyQt4.QtCore.Qt.ShiftModifier:
+                shift_offset = BOX_BUTTONS_SEED_SHIFT_OFFSET
+            elif modifiers == PyQt4.QtCore.Qt.ControlModifier:
+                # this make seed_mark = 0 when left button is pressed
+                shift_offset = -1
+
+            #     print('Control+Click')
+            # elif modifiers == (QtCore.Qt.ControlModifier |
+            #                    QtCore.Qt.ShiftModifier):
+            # print('Control+Shift+Click')
+            self.seed_mark = self.box_buttons[event.button()] + shift_offset
             self.last_position = self.gridPosition(event.pos())
 
         elif event.button() == Qt.MiddleButton:
@@ -534,8 +563,8 @@ class QTSeedEditor(QDialog):
         combo_view.addItems(combo_view_options)
 
         # buttons
-        btn_quit = QPushButton("Return", self)
-        btn_quit.clicked.connect(self.quit)
+        self.btn_quit = QPushButton("Return", self)
+        self.btn_quit.clicked.connect(self.quit)
 
         combo_dmask = QComboBox(self)
         combo_dmask.activated.connect(self.changeMask)
@@ -548,18 +577,28 @@ class QTSeedEditor(QDialog):
 
         self.status_bar = QStatusBar()
 
+        self.seeds_copy = None
         vopts = []
         vmenu = []
         appmenu = []
         if mode == 'seed' and self.mode_fun is not None:
             btn_recalc = QPushButton("Recalculate", self)
             btn_recalc.clicked.connect(self.recalculate)
+            self.btn_save= QPushButton("Save seeds", self)
+            self.btn_save.clicked.connect(self.saveload_seeds)
+            btn_s2b= QPushButton("Seg. to bckgr.", self)
+            btn_s2b.clicked.connect(self.seg_to_background_seeds)
+            btn_s2f= QPushButton("Seg. to forgr.", self)
+            btn_s2f.clicked.connect(self.seg_to_foreground_seeds)
             appmenu.append(QLabel('<b>Segmentation mode</b><br><br><br>' +
                                   'Select the region of interest<br>' +
                                   'using the mouse buttons:<br><br>' +
                                   '&nbsp;&nbsp;<i>left</i> - inner region<br>' +
                                   '&nbsp;&nbsp;<i>right</i> - outer region<br><br>'))
             appmenu.append(btn_recalc)
+            appmenu.append(self.btn_save)
+            appmenu.append(btn_s2f)
+            appmenu.append(btn_s2b)
             appmenu.append(QLabel())
             self.volume_label = QLabel('Volume:\n  unknown')
             appmenu.append(self.volume_label)
@@ -586,6 +625,7 @@ class QTSeedEditor(QDialog):
             vopts.append(combo_contour)
 
 
+
         if mode == 'mask':
             btn_recalc_mask = QPushButton("Recalculate mask", self)
             btn_recalc_mask.clicked.connect(self.updateMaskRegion_btn)
@@ -686,7 +726,7 @@ class QTSeedEditor(QDialog):
                 vbox_app.addWidget(ii)
 
         vbox_app.addStretch(1)
-        vbox_app.addWidget(btn_quit)
+        vbox_app.addWidget(self.btn_quit)
 
         hbox.addLayout(vbox_left)
         hbox.addWidget(self.get_line('v'))
@@ -732,6 +772,9 @@ class QTSeedEditor(QDialog):
 
         QDialog.__init__(self)
 
+        self.BACKGROUND_NOMODEL_SEED_LABEL = 4
+        self.FOREGROUND_NOMODEL_SEED_LABEL = 3
+
         self.mode = mode
         self.mode_fun = modeFun
 
@@ -743,7 +786,7 @@ class QTSeedEditor(QDialog):
         self.volume_unit = volume_unit
 
         self.last_view_position = {}
-        for jj, ii in enumerate(VIEW_TABLE.iterkeys()):
+        for jj, ii in enumerate(VIEW_TABLE.keys()):
             if viewPositions is None:
                 viewpos = img.shape[VIEW_TABLE[ii][-1]] / 2
 
@@ -764,8 +807,8 @@ class QTSeedEditor(QDialog):
 
         # masked data - has information about which data were removed
         # 1 == enabled, 0 == deleted
-        # How to return: 
-        #       editorDialog.exec_() 
+        # How to return:
+        #       editorDialog.exec_()
         #       masked_data = editorDialog.masked
         self.masked = np.ones(self.img.shape, np.int8)
 
@@ -776,7 +819,6 @@ class QTSeedEditor(QDialog):
         # set seeds
         if seeds is None:
             self.seeds = np.zeros(self.img.shape, np.int8)
-
         else:
             self.seeds = seeds
 
@@ -867,33 +909,33 @@ class QTSeedEditor(QDialog):
 
     def maskRegion(self):
         self.masked[self.contours == 0] = 0
-        
+
         self.img[self.contours != 2] = self.img_min_val
         self.contours.fill(0)
         self.contours_old = self.contours.copy()
         self.seeds.fill(0)
         self.selectSlice(self.actual_slice)
-        
+
     def maskAddSelection(self):
         self.updateMaskRegion()
         if self.contours is None:
             return
-        
+
         self.contours[self.contours == 1] = 2
         self.contours_old = self.contours.copy()
         self.seeds.fill(0)
         self.selectSlice(self.actual_slice)
-        
+
     def maskRemoveSelection(self):
         self.updateMaskRegion()
         if self.contours is None:
             return
-            
+
         self.contours[self.contours == 1] = 0
         self.contours_old = self.contours.copy()
         self.seeds.fill(0)
         self.selectSlice(self.actual_slice)
-        
+
     def maskSelectAll(self):
         self.updateMaskRegion()
 
@@ -905,20 +947,20 @@ class QTSeedEditor(QDialog):
         self.seeds[-1][0][-1] = 1
         self.seeds[-1][-1][0] = 1
         self.seeds[-1][-1][-1] = 1
-        
+
         self.updateMaskRegion()
         self.selectSlice(self.actual_slice)
-        
+
     def resetSelection(self):
         self.updateMaskRegion()
         if self.contours is None:
             return
-            
+
         self.contours.fill(0)
         self.contours_old = self.contours.copy()
         self.seeds.fill(0)
         self.selectSlice(self.actual_slice)
-        
+
     def resetSeads(self):
         self.seeds.fill(0)
         if self.contours is not None:
@@ -993,6 +1035,15 @@ class QTSeedEditor(QDialog):
         return self.contours
 
     def setContours(self, contours):
+        """
+        store segmentation
+        :param contours: segmentation
+        :return: Nothing
+        """
+        """
+        :param contours:
+        :return:
+        """
         self.contours = contours
         self.contours_aview = self.contours.transpose(self.act_transposition)
 
@@ -1114,7 +1165,7 @@ class QTSeedEditor(QDialog):
 
     def cropUpdate(self, img):
 
-        for ii in VIEW_TABLE.iterkeys():
+        for ii in VIEW_TABLE.keys():
             self.last_view_position[ii] = 0
         self.actual_slice = 0
 
@@ -1205,6 +1256,43 @@ class QTSeedEditor(QDialog):
 
         self.cropUpdate(self.img)
 
+    def seg_to_background_seeds(self, event):
+        self.saveSliceSeeds()
+        self.seeds[self.seeds < 3] = 0
+
+        from PyQt4.QtCore import pyqtRemoveInputHook
+        # pyqtRemoveInputHook()
+        # import ipdb; ipdb.set_trace()
+        self.seeds[(self.contours == 1) & (self.seeds < 3)] = self.BACKGROUND_NOMODEL_SEED_LABEL
+        self.contours[...] = 0
+
+    def seg_to_foreground_seeds(self, event):
+        self.saveSliceSeeds()
+        self.seeds[self.seeds < 3] = 0
+        # from PyQt4.QtCore import pyqtRemoveInputHook
+        # pyqtRemoveInputHook()
+        # import ipdb; ipdb.set_trace()
+        self.seeds[(self.contours == 1) & (self.seeds < 3)] = self.FOREGROUND_NOMODEL_SEED_LABEL
+        self.contours[...] = 0
+
+    def saveload_seeds(self, event):
+        if self.seeds_copy is None:
+            self.seeds_copy = self.seeds.copy()
+            self.seeds[...] = 0
+            # print "save"
+            # from PyQt4.QtCore import pyqtRemoveInputHook
+            # pyqtRemoveInputHook()
+            # import ipdb; ipdb.set_trace()
+            self.btn_save.setText("Load seeds")
+        else:
+            # from PyQt4.QtCore import pyqtRemoveInputHook
+            # pyqtRemoveInputHook()
+            # import ipdb; ipdb.set_trace()
+            self.seeds[self.seeds_copy > 0] = self.seeds_copy[self.seeds_copy > 0]
+            self.seeds_copy = None
+            self.btn_save.setText("Save seeds")
+
+
     def recalculate(self, event):
         self.saveSliceSeeds()
         if np.abs(np.min(self.seeds) - np.max(self.seeds)) < 2:
@@ -1266,8 +1354,11 @@ class QTSeedEditor(QDialog):
 
 def gen_test():
     test = {}
-    test['data'] = np.zeros((10,10,4), dtype=np.uint8)
+    # test['data'] = np.zeros((10,10,4), dtype=np.uint8)
+    test['data'] = (np.random.rand(4, 10, 9) * 100).astype(np.uint8)
+
     test['voxelsize_mm'] = (2, 2, 2.5)
+    test['segdata'] = None
 
     return test
 
@@ -1316,11 +1407,14 @@ def main():
 
     app = QApplication(sys.argv)
     pyed = QTSeedEditor(dataraw['data'],
-                        seeds=dataraw['segdata'],
+                        # seeds=dataraw['segdata'],
                         mode=options.mode,
                         voxelSize=dataraw['voxelsize_mm'])
+    pyed.changeC(50)
+    pyed.changeW(50)
+    pyed.exec_()
 
-    sys.exit(app.exec_())
+    # sys.exit(app.exec_())
 
 if __name__ == "__main__":
     main()
